#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the PyUtilib README.txt file.
#  _________________________________________________________________________


import sys

_old_stdout = []
_old_stderr = []

def setup_redirect(ofile):
    """ Redirect stdout and stderr to a specified file """
    global _old_stdout
    global _old_stderr
    _old_stdout.append(sys.stdout)
    _old_stderr.append(sys.stderr)
    sys.stderr = _Redirecter(ofile)
    sys.stdout = sys.stderr

def reset_redirect():
    """ Reset redirection to use standard stdout and stderr """
    global _old_stdout
    global _old_stderr
    if not _old_stdout is []:
        sys.stdout.close()
        sys.stdout = _old_stdout.pop()
        sys.stderr = _old_stderr.pop()

#
# A class used to manage the redirection of IO.  The sys.stdout and 
# sys.stderr values are set to an instance of this class.
#
class _Redirecter:

    def __init__(self, ofile):
        """ Constructor. """
        self.ofile = ofile
        self._out = open(ofile,"w")

    def write(self, s):
        """ Write an item. """
        self._out.write(s)

    def flush(self):
        """ Flush the output. """
        self._out.flush()

    def close(self):
        """ Close the stream. """
        self._out.close()

